<?php

/**
 * Main Widget Factory for the Advanced Sidebar Menu Pro plugin
 *
 * @author Mat Lipe
 *
 * @since  1.0
 *
 * @uses   constructed at the bottom of this file
 */
class Advanced_Sidebar_Menu_Pro_Factory {
	const CSS_SELECTOR_APPEND = '-wrap';

	protected $widget_count = 0;

	const CATEGORY_TYPE = 'category';
	const PAGE_TYPE     = 'page';


	protected function hook() {
		add_action( 'advanced-sidebar-menu/widget/page/after-form', array( $this, 'add_documentation_link' ), 99, 4 );
		add_action( 'advanced-sidebar-menu/widget/category/after-form', array(
			$this,
			'add_documentation_link',
		), 99, 4 );

		add_action( 'advanced-sidebar-menu/widget/page/before-form', array( $this, 'wrap_widget_form' ), 1, 2 );
		add_action( 'advanced-sidebar-menu/widget/page/after-form', array( $this, 'close_div' ), 99, 0 );
		add_action( 'advanced-sidebar-menu/widget/category/before-form', array( $this, 'wrap_widget_form' ), 1, 2 );
		add_action( 'advanced-sidebar-menu/widget/category/after-form', array( $this, 'close_div' ), 99, 0 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/before-form', array(
			$this,
			'wrap_widget_form',
		), 1, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/after-form', array( $this, 'close_div' ), 99, 0 );

		add_filter( 'advanced_sidebar_menu_page_widget_output', array( $this, 'wrap_page_widget' ), 10, 4 );
		add_filter( 'advanced_sidebar_menu_category_widget_output', array( $this, 'wrap_widget_on_render' ), 10, 3 );
		add_filter( 'advanced-sidebar-menu/widget/navigation-menu/output', array(
			$this,
			'wrap_widget_on_render',
		), 10, 3 );

		add_action( 'widgets_init', array( $this, 'register_widgets' ) );

	}


	/**
	 * @action widgets_init 10 0
	 *
	 * @since  3.3.0
	 *
	 * @return void
	 */
	public function register_widgets() {
		register_widget( 'Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu' );
	}


	/**
	 * Are we a category widget or a page widget?
	 *
	 * @param WP_Widget $widget
	 *
	 * @return string - 'page', 'category'
	 */
	public static function get_widget_type( WP_Widget $widget ) {
		if ( 'advanced_sidebar_menu_category' === $widget->id_base ) {
			return self::CATEGORY_TYPE;
		}

		return self::PAGE_TYPE;
	}


	/**
	 * Wrap the admin widget so we can target with JS
	 *
	 * @param array      $instance
	 * @param \WP_Widget $widget
	 *
	 * @since 3.2.0
	 *
	 * @return void
	 */
	public function wrap_widget_form( array $instance, $widget ) {
		// because new widgets will have an __i__ which gets changed by js
		// this is the only way to track new widgets.
		$this->widget_count ++;
		$id = $widget->id_base . '-' . $this->widget_count;
		?>
		<div id="<?php echo esc_attr( $id ); ?>">
		<!-- done this way to handle the reload of the widget after hitting save -->
		<script type="text/javascript">
			//new widgets being added
			if (typeof (asm_pro) !== 'undefined' && typeof (asm_pro.init) === 'function') {
				asm_pro.init('#<?php echo esc_js( $id ); ?>');
			} else {
				//existing widgets
				//we don't trigger the init for the background __i__ widget on page load
				//because the color pickers will be broken when adding a new widget
				<?php if ( '__i__' !== $widget->number ) { ?>
				jQuery(function () {
					if (typeof (asm_pro) !== 'undefined') {
						asm_pro.init('#<?php echo esc_js( $id ); ?>');
					}
				});
				<?php } ?>
			}
		</script>
		<?php
	}


	/**
	 * @since 3.2.0
	 *
	 * @return void
	 */
	public function close_div() {
		?>
		</div>
		<?php
	}


	/**
	 * Wrap widget so we can target it directly with js and css and
	 * not worry about themes which do not declare the widget id
	 *
	 * @notice Ugh! THE page widget has different args so it needs it's own method
	 *
	 * @since  1.8.0
	 *
	 * @action advanced_sidebar_menu_page_widget_output 10
	 * @action advanced_sidebar_menu_category_widget_output 10
	 *
	 * @param string   $output
	 * @param \WP_Post $post
	 * @param  []       $args
	 * @param  []       $instance
	 *
	 * @return string
	 */
	public function wrap_page_widget( $output, $post, $args ) {
		$output = '<div id="' . $args['widget_id'] . self::CSS_SELECTOR_APPEND . '" data-js="' . $args['widget_id'] . self::CSS_SELECTOR_APPEND . '">' . $output . '</div>';

		return $output;
	}


	/**
	 * Wrap widget so we can target it directly with js and css and
	 * not worry about themes which do not declare the widget id
	 *
	 * @notice Ugh! THE page widget has different args so it needs it's own method
	 *
	 * @since  3.3.0
	 *
	 * @action advanced_sidebar_menu_category_widget_output 10
	 * @action
	 *
	 * @param string $output
	 * @param  []     $args
	 * @param  []     $instance
	 *
	 * @return string
	 */
	public function wrap_widget_on_render( $output, $args ) {
		$output = '<div id="' . $args['widget_id'] . self::CSS_SELECTOR_APPEND . '" data-js="' . $args['widget_id'] . self::CSS_SELECTOR_APPEND . '">' . $output . '</div>';

		return $output;
	}


	/**
	 * Add a link to documentation
	 *
	 * @param $instance
	 * @param $widget
	 *
	 * @since 3.2.0
	 *
	 * @return void
	 */
	public function add_documentation_link( $instance, $widget ) {
		?>
		<p style="text-align: right">
			<a href="<?php echo self::get_docs_url( $widget ); ?>" target="blank">
				<?php esc_html_e( 'widget documentation', 'advanced-sidebar-menu' ); ?>
			</a>
		</p>
		<?php
	}


	/**
	 *
	 * @param \WP_Widget $widget
	 *
	 * @static
	 *
	 * @return string
	 */
	public static function get_docs_url( $widget ) {
		$type = self::get_widget_type( $widget );
		$url  = 'https://matlipe.com/advanced-sidebar-menu/advanced-sidebar-menu-pro-widget-docs/';

		if ( self::CATEGORY_TYPE === $type ) {
			return $url . '#category-widget';
		}

		return $url . '#page-widget';

	}

	// ********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}

