<?php


class Advanced_Sidebar_Menu_Pro__Meta__Title {
	const EXCLUDE_PAGE = 'advanced-sidebar-menu-pro/exclude-this-page';
	const LINK_TEXT    = 'asm-link-text';

	const NONCE = 'advanced-sidebar-menu-pro/title-nonce';
	const CACHE = 'advanced-sidebar-menu-pro/excluded-pages';


	protected function hook() {
		add_action( 'do_meta_boxes', array( $this, 'createMetaBox' ) );
		add_action( 'save_post', array( $this, 'saveMeta' ) );
		add_action( 'advanced_sidebar_menu_widget_pre_render', array( $this, 'start_filter' ) );
		add_filter( 'advanced_sidebar_menu_page_widget_output', array( $this, 'remove_filter' ) );
		add_filter( 'advanced_sidebar_menu_excluded_pages', array( $this, 'add_excluded_by_meta_box_pages' ), 9, 1 );
	}


	public function start_filter() {
		add_filter( 'the_title', array( $this, 'change_link_title' ), 99, 2 );
	}


	public function remove_filter( $out ) {
		remove_filter( 'the_title', array( $this, 'change_link_title' ), 99 );

		return $out;
	}


	/**
	 * Exclude pages which are checking in the meta box to be excluded
	 *
	 * @param [] $excluded
	 *
	 * @return array
	 */
	public function add_excluded_by_meta_box_pages( $excluded ) {
		$to_be_excluded = wp_cache_get( self::CACHE );
		if ( false === $to_be_excluded ) {
			$to_be_excluded = get_posts(
				array(
					'post_type'              => 'page',
					'nopaging'               => true,
					'fields'                 => 'ids',
					'update_post_meta_cache' => false,
					'update_post_term_cache' => false,
					'suppress_filters'       => false,
					'meta_query'             => array(
						array(
							'key'     => self::EXCLUDE_PAGE,
							'compare' => 'EXISTS',
						),
					),
				)
			);
			if ( empty( $to_be_excluded ) ) {
				$to_be_excluded = array();
			}
			wp_cache_set( self::CACHE, $to_be_excluded );
		}
		$excluded = array_merge( $excluded, $to_be_excluded );

		return array_unique( $excluded );
	}


	/**
	 * Use the value set in the meta box for the title
	 *
	 * @param string $title - the original title.
	 * @param null   $post_id - null does not really work here but there are other plugins which do not
	 *                        pass a second argument to the 'the_title' filter which throw fatals on
	 *                        this argument.
	 *
	 * @see https://matlipe.com/plugin-support/plugin-conflict-with-nextend-smart-slider-plugin/comment-page-1
	 *
	 * @return mixed
	 */
	public function change_link_title( $title, $post_id = null ) {
		$text = get_post_meta( $post_id, self::LINK_TEXT, true );
		if ( empty( $text ) ) {
			return $title;
		}

		return $text;
	}


	protected function flush_cache() {
		wp_cache_delete( self::CACHE );
	}


	public function createMetaBox() {
		add_meta_box(
			'asm-custom-title',
			esc_html__( 'Advanced Sidebar Menu', 'advanced-sidebar-menu' ),
			array(
				$this,
				'metaBoxOutput',
			),
			'page',
			'side',
			'high'
		);
	}


	public function saveMeta( $post_id ) {
		if ( ! empty( $_POST[ self::NONCE ] ) && wp_verify_nonce( $_POST[ self::NONCE ], self::NONCE ) ) { // phpcs:ignore

			$this->flush_cache();

			if ( empty( $_POST[ self::LINK_TEXT ] ) ) {
				delete_post_meta( $post_id, self::LINK_TEXT );
			} else {
				update_post_meta( $post_id, self::LINK_TEXT, $_POST[ self::LINK_TEXT ] ); // phpcs:ignore
			}

			if ( empty( $_POST[ self::EXCLUDE_PAGE ] ) ) {
				delete_post_meta( $post_id, self::EXCLUDE_PAGE );
			} else {
				update_post_meta( $post_id, self::EXCLUDE_PAGE, 1 );
			}
		}
	}


	public function metaBoxOutput( $post ) {
		wp_nonce_field( self::NONCE, self::NONCE );
		?>
		<p>
			<?php esc_html_e( "Title that will be used for this page's link when it is displayed in the Advanced Sidebar Menus.", 'advanced-sidebar-menu' ); ?>
			<label class="screen-reader-text" for="<?php echo esc_attr( self::LINK_TEXT ); ?>">
				<?php esc_html_e( "Page widget link's text", 'advanced-sidebar-menu' ); ?>
			</label>
			<input name="<?php echo esc_attr( self::LINK_TEXT ); ?>" type="text" class="widefat" id="asm-link-text" value="<?php echo esc_attr( get_post_meta( $post->ID, self::LINK_TEXT, true ) ); ?>" placeholder="<?php echo esc_attr( $post->post_title ); ?>">

		</p>
		<p>
			<label class="screen-reader-text" for="<?php echo esc_attr( self::EXCLUDE_PAGE ); ?>">
				<?php esc_html_e( 'Check to exclude page from menus', 'advanced-sidebar-menu' ); ?>
			</label>
			<input name="<?php echo esc_attr( self::EXCLUDE_PAGE ); ?>" type="checkbox" value="1" id="asm-exclude-page" <?php checked( (bool) get_post_meta( $post->ID, self::EXCLUDE_PAGE, true ) ); ?>
			"/>
			<?php esc_html_e( 'Exclude this page from the Advanced Sidebar Menus', 'advanced-sidebar-menu' ); ?>.
		</p>
		<?php
	}


	// ********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}
