<?php

/**
 * Walker Display Posts
 *
 * Add posts under each category if the option is selected to do so
 *
 * @author Mat Lipe
 * @since  3.2.0
 */
class Advanced_Sidebar_Menu_Pro__Walkers__Display_Posts extends Walker_Category {
	/**
	 * The widget instance
	 *
	 * @var array
	 */
	protected $instance;

	/**
	 * Posts assigned to the category
	 *
	 * @var \WP_Post[]
	 */
	protected $children;


	/**
	 * Advanced_Sidebar_Menu_Pro__Walkers__Display_Posts constructor.
	 *
	 * @param array $instance - Current widget settings.
	 */
	public function __construct( array $instance ) {
		$this->instance = $instance;
	}


	/**
	 * Add the has_children class to an item which has post children
	 *
	 * @param array $css_classes - Current item classes.
	 *
	 * @return array
	 */
	public function add_has_children_class( $css_classes ) {
		$css_classes[] = 'has_children';

		return $css_classes;
	}

	/**
	 * Add the current-cat-parent class to items which have a child post
	 * which is the current post
	 *
	 * @param array $css_classes - Current item classes.
	 *
	 * @return array
	 */
	public function add_current_cat_parent_class( $css_classes ) {
		$css_classes[] = 'current-cat-parent';

		return $css_classes;
	}


	/**
	 * Add the necessary filters and retrieve posts for the current list item.
	 * Then call the parent `start_el` like normal.
	 *
	 * @param string $output - Current list output.
	 * @param object $term - Current category or term.
	 * @param int    $depth - What level of the menu we are on.
	 * @param array  $args - Additional arguments passed to wp_list_categories.
	 * @param int    $id - Id of the current item.
	 *
	 * @return void
	 */
	public function start_el( &$output, $term, $depth = 0, $args = array(), $id = 0 ) {
		$_args = apply_filters( 'advanced-sidebar-menu-pro/walker/display-posts/get-post-args', array(
			'post_type'        => $this->instance[ Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Display_Posts::POST_TYPE ],
			'numberposts'      => $this->instance[ Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Display_Posts::LIMIT ],
			'orderby'          => 'title',
			'order'            => 'ASC',
			'suppress_filters' => false,
			'tax_query'        => array(
				array(
					'taxonomy'         => $term->taxonomy,
					'terms'            => $term->term_id,
					'include_children' => false,
				),
			),
		), $term, $depth, $args, $id );

		$this->children = get_posts( $_args );

		if ( ! empty( $this->children ) ) {
			add_filter( 'category_css_class', array( $this, 'add_has_children_class' ) );
			// add the current-cat-parent class to the parent term of the current post.
			$current_post = is_single() ? get_post() : null;
			if ( ! empty( $current_post ) ) {
				$ids = wp_list_pluck( $this->children, 'ID' );
				if ( in_array( $current_post->ID, $ids, false ) ) { //phpcs:disable
					add_filter( 'category_css_class', array( $this, 'add_current_cat_parent_class' ) );
				}
			}
		}

		parent::start_el( $output, $term, $depth, $args, $id );
	}


	/**
	 *
	 * @param string $output
	 * @param object $term
	 * @param int    $depth
	 * @param array  $args
	 *
	 * @return void
	 */
	public function end_el( &$output, $term, $depth = 0, $args = array() ) {
		remove_filter( 'category_css_class', array( $this, 'add_has_children_class' ) );
		remove_filter( 'category_css_class', array( $this, 'add_current_cat_parent_class' ) );
		$current_post = is_single() ? get_post() : null;

		if ( ! empty( $this->children ) ) {
			ob_start();
			?>
			<ul class="children">
				<?php
				foreach ( $this->children as $_post ) {
					$class = 'child-post cat-item';
					if ( ! empty( $current_post ) && $current_post->ID === $_post->ID && is_single( $current_post ) ) {
						$class .= ' current-cat';
					}
					?>
					<li class="<?php echo esc_attr( $class ); ?>">
						<a
							href="<?php echo esc_url( get_permalink( $_post ) ); ?>"
							title="<?php echo esc_attr( $_post->post_title ); ?>">
							<?php echo get_the_title( $_post ) ?>
						</a>
						<?php
						// @link https://matlipe.com/plugin-support/show-child-posts-under-parent-posts/
						do_action( 'advanced-sidebar-menu-pro/walker/display-posts/post-element', $_post, $this->instance, $this );
						?>
					</li>
					<?php
				}
				?>
			</ul>
			<?php
			$output .= ob_get_clean();
		}

		parent::end_el( $output, $term, $depth, $args );
	}
}
