<?php

/**
 * Current Page Parent Only Option for Page Widget
 *
 * @uses   Automatically called in by the Advanced_Sidebar_Menu_Pro::add_documentation_link()
 *
 * @author Mat Lipe <mat@matlipe.com>
 */
class Advanced_Sidebar_Menu_Pro__Widget_Options__Page__Current_Page_Parent_Only extends Advanced_Sidebar_Menu_Pro__Widget_Options__Fields {

	const NAME = 'parent_only';

	const DISPLAY_SIBLINGS_CHILD_PAGE       = 'parent_only/display-sibling/child-page';
	const DISPLAY_SIBLINGS_NO_GRANDCHILDREN = 'parent_only/display-sibling/no-grandchildren';


	/**
	 * Add the actions.
	 *
	 * @return void
	 */
	protected function hook() {
		add_filter( 'advanced-sidebar-menu/list-pages/first-level-child-pages', array(
			$this,
			'exclude_non_ancestor_pages',
		), 1, 3 );

		add_action( 'advanced-sidebar-menu/widget/page/right-column', array( $this, 'render_fields' ), 15, 2 );

	}


	/**
	 * Render the widget fields.
	 *
	 * @param array                                 $instance - Widget settings.
	 * @param Advanced_Sidebar_Menu__Widget__Widget $widget   - Widget Class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance[ self::NAME ] = isset( $instance[ self::NAME ] ) ? $instance[ self::NAME ] : '';

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::NAME, self::DISPLAY_SIBLINGS_CHILD_PAGE ); ?>
				<label><?php esc_html_e( "Display current page's parents only", 'advanced-sidebar-menu' ); ?>
				</label>
				<br/>
				<span class="description">
					<em>
						<?php esc_html_e( 'Only display the current page and any parents, grandparents or children of the current page.', 'advanced-sidebar-menu' ); ?>
					</em>
				</span>
			</p>
			<div <?php $widget->hide_element( self::NAME, self::DISPLAY_SIBLINGS_CHILD_PAGE ); ?>>

				<p>
					<?php $widget->checkbox( self::DISPLAY_SIBLINGS_CHILD_PAGE, self::DISPLAY_SIBLINGS_NO_GRANDCHILDREN ); ?>
					<label>
						<?php esc_html_e( 'Display siblings when on a child page', 'advanced-sidebar-menu' ); ?>
					</label>
					<br/>
					<span class="description">
				<em><?php esc_html_e( 'Display siblings of the current page when on a child page.', 'advanced-sidebar-menu' ); ?></em>
			</span>
				</p>
				<p <?php $widget->hide_element( self::DISPLAY_SIBLINGS_CHILD_PAGE, self::DISPLAY_SIBLINGS_NO_GRANDCHILDREN ); ?>>
					<?php $widget->checkbox( self::DISPLAY_SIBLINGS_NO_GRANDCHILDREN ); ?>
					<label>
						<?php esc_html_e( 'Display siblings when there are no grandchildren', 'advanced-sidebar-menu' ); ?>
					</label>
					<br/>
					<span class="description">
						<em><?php esc_html_e( 'Display siblings of the current page when on a child page that has no children.', 'advanced-sidebar-menu' ); ?></em>
					</span>
				</p>
			</div>
		</div>
		<?php
	}


	/**
	 * Exclude any non ancestor child pages by removing them from the
	 * child_pages array
	 *
	 * @param WP_Post[]                        $pages      - List of current pages pre filter.
	 * @param Advanced_Sidebar_Menu_List_Pages $list_pages - List_Pages class.
	 * @param Advanced_Sidebar_Menu_Menus_Page $menu       - Menu class.
	 *
	 * @return array
	 */
	public function exclude_non_ancestor_pages( array $pages, $list_pages, $menu ) {
		if ( ! $this->checked( self::NAME, $menu->instance ) || 0 === $menu->get_current_post()->post_parent ) {
			return $pages;
		}

		$current_page = $menu->get_current_post();

		// Display siblings when on a child page.
		if ( $this->checked( self::DISPLAY_SIBLINGS_CHILD_PAGE, $menu->instance ) && $current_page->post_parent === $menu->get_top_parent_id() ) {
			return $pages;
		}

		// Display siblings when no grandchild pages available.
		if ( $this->checked( self::DISPLAY_SIBLINGS_NO_GRANDCHILDREN, $menu->instance ) && $current_page->post_parent === $menu->get_top_parent_id() ) {
			$children = $list_pages->get_child_pages( $current_page->ID );
			if ( empty( $children ) ) {
				return $pages;
			}
		}

		// Default behavior with no sub options checked.
		foreach ( $pages as $k => $_post ) {
			if ( ! $list_pages->is_current_page_ancestor( $_post->ID ) ) {
				unset( $pages[ $k ] );
			}
		}

		return $pages;
	}


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}
